# Program to take mbspect input parameters, run them through miriad and output
# measured parameters in an AGES-compatible format.
# The user can set the filename manually on line 54. To use the filename given
# in the input files instead, comment out lines 53 and 54.
# The velocity range of the cube must be specified on lines 18 and 19. The
# script will centre the velocity range on the source and truncate it if the
# 2,000 km/s of required baseline would exceed the limits.

import os
import string
from string import *
import re

# Convert the FileList into a internal Python list
List = open('FileList.txt','r')

# Set minimum and maximum velocity range of the cube
minxr = 100.0
maxxr = 20000.0

Files=[]

for line in List:
	# Splitting by \n removes line breaks
	Files.append(line.split('\n')[0])

List.close()

for j in range(0,len(Files)):
	commands=[]		# This will be a string array containing all the input parameters

	Input = open(Files[j],'r')
	print 'Processing ',Files[j]

	# We want to manually set the filename and set the xrange based on the profile
	# So, first go through file to get profile
	for line in Input:
		if 'profile=' in line:
			# First split by the comma to get profile=vmin
			half = line.split(',')[0]
			# Now split by equals to get vmin itself
			vmin = half.split('=')[1]
			# Can get vmax directly
			vmax = line.split(',')[1]
			
			vcent = (float(vmax) + float(vmin)) / 2.0
	
	Input.seek(0)	
	
	for line in Input:
		thisline = line
		# Manually set input file. Comment the next two lines to use the filename in the input parameter files.
		if 'in=' in thisline:
			thisline = str('in=AGESVC2_100-5000.mir')+'\n'
			
		# Set xrange based on vcent
		if 'xrange=' in thisline:
			xmin = vcent-2000.0
			xmax = vcent+2000.0
			print 'Default xmin,xmax based on line centre :',xmin,xmax

			if xmin < minxr:
				xmin = minxr

			if xmax > maxxr:
				xmax = maxxr

			print 'Corrected values of xmin,xmax :',xmin,xmax
			thisline = 'xrange='+str(xmin)+','+str(xmax)+'\n'			
			
		commands.append(thisline)
		
		# Save the name of the log file as it will be needed later
		if 'log' in line:
			logfile = line.split('=')[1]
			# Annoyingly, have to use weired regular expressions to
			# remove all whitespace
			logfile = re.sub(r'\s', '',logfile)

	Input.close()

	comstring = 'mbspect '
	
	# Get all the input parameters into a single, one-line string
	for i in range(0,len(commands)):
		# Annoying regular expressions syntax to remove all whitespace
		comstring = comstring + re.sub(r'\s', '',commands[i])	+ ' '

	# Write this string into a file miriad can accept as a command	
	ComFile = open('MiriadInputs','w')

	ComFile.write(comstring+'\n')
	ComFile.write('exit')

	ComFile.close()
	

	# Run mbspect with the command file just produced, save the output
	os.system('miriad < MiriadInputs > TempOutput.txt')

	os.system('rm MiriadInputs')

	# Now we have all 3 files, but the inputs and mbspect output need to be properly formatted.
	# First produce a commented version of the input file.

	Temp1 = open('TempInputs.txt','w')
	Temp1.write('# INPUT PARAMETERS'+'\n')

	for i in range(0,len(commands)):
		thing = str('#') + str(commands[i])
		Temp1.write(thing)

	Temp1.close()

	# Next, trim and comment the msbpect outputs
	Output = open('TempOutput.txt','r')
	FormatOutput = open('TempOutput2.txt','w')

	i = -1


	# If position fitting fails there will be nothing in the TempOutput file apart
	# from the input parameters. So count number of actual output lines.
	usefullines = 0
	for line in Output:	
		i = i + 1
		
		# Skip the first 4 lines, and omit the final line which tells miriad
		# to exit.
		if (i <=3 or ('exit' in line)) and ('#' not in line):
			pass
		else:
			usefullines = usefullines + 1
			if line.startswith('#'):
				FormatOutput.write(line)
			else:
				thing = str('#')+str(line)
				FormatOutput.write(thing)
				
	# If there was no output, inform the user and call a halt before any files have
	# been altered (except the Temp files, which can safely be removed)
	if usefullines	== 0:
		print 'Error running file ',Files[j]
		print 'Program halted, nothing altered. Correct the file and try again.'
		print 'Also remove all files before this one in FileList !'		

	FormatOutput.close()

	# Now everything is properly formatted. Just need to join everything together !

	Complete=[]
	FirstFile = open('TempInputs.txt','r')
	SecondFile= open('TempOutput2.txt','r')
	ThirdFile =open(logfile,'r')

	for line in FirstFile:
		Complete.append(line)
	for line in SecondFile:
		Complete.append(line)
	for line in ThirdFile:
		Complete.append(line)

	FirstFile.close()
	SecondFile.close()
	ThirdFile.close()
	

	filename = logfile.split('.txt')[0]+str('spec')+str('.txt')

	Final = open(filename,'w')

	for i in range(0,len(Complete)):
		Final.write(Complete[i])

	Final.close()

	os.system('rm Temp*')
	
	# Remove the input files and log files produced.
	os.remove(Files[j])
	try:
		os.remove(logfile)
	except:
		pass
